
local ipairs, pairs, tonumber, setmetatable, type, tostring = ipairs, pairs, tonumber, setmetatable, type, tostring

local math             = require("math")
local table            = require("table")
local os	       = require("os")	
local string	       = require("jive.utils.string")
local debug	       = require("jive.utils.debug")

local oo               = require("loop.simple")

local Applet           = require("jive.Applet")
local Font             = require("jive.ui.Font")
local Framework        = require("jive.ui.Framework")
--local Group            = require("jive.ui.Group")
local Icon             = require("jive.ui.Icon")
local Canvas           = require("jive.ui.Canvas")
--local Choice           = require("jive.ui.Choice")
local Label            = require("jive.ui.Label")
--local RadioButton      = require("jive.ui.RadioButton")
--local RadioGroup       = require("jive.ui.RadioGroup")
--local SimpleMenu       = require("jive.ui.SimpleMenu")
local Surface          = require("jive.ui.Surface")
local Tile             = require("jive.ui.Tile")
local Window           = require("jive.ui.Window")
local SnapshotWindow   = require("jive.ui.SnapshotWindow")

local Player           = require("jive.slim.Player")
                       
local datetime         = require("jive.utils.datetime")

local appletManager	= appletManager
local jiveMain          = jiveMain
local jnt               = jnt

local LAYER_FRAME            = jive.ui.LAYER_FRAME
local LAYER_CONTENT_ON_STAGE = jive.ui.LAYER_CONTENT_ON_STAGE

local LAYOUT_NORTH           = jive.ui.LAYOUT_NORTH
local LAYOUT_EAST            = jive.ui.LAYOUT_EAST
local LAYOUT_SOUTH           = jive.ui.LAYOUT_SOUTH
local LAYOUT_WEST            = jive.ui.LAYOUT_WEST
local LAYOUT_CENTER          = jive.ui.LAYOUT_CENTER
local LAYOUT_NONE            = jive.ui.LAYOUT_NONE

local WH_FILL                = jive.ui.WH_FILL


module(..., Framework.constants)
oo.class(_M, Applet)

-- Define useful variables for this skin
local fontpath = "fonts/"
local FONT_NAME = "FreeSans"
local BOLD_PREFIX = "Bold"

local function _imgpath(self)
	return "applets/WordClock/images/" .. self.skinName .. "/"
end

-- define a local function that makes it easier to set fonts
local function _font(fontSize)
	return Font:load(fontpath .. FONT_NAME .. ".ttf", fontSize)
end

-- define a local function that makes it easier to set bold fonts
--local function _boldfont(fontSize)
--	return Font:load(fontpath .. FONT_NAME .. BOLD_PREFIX .. ".ttf", fontSize)
--end

-- defines a new style that inherrits from an existing style
local function _uses(parent, value)
	if parent == nil then
		log:warn("nil parent in _uses at:\n", debug.traceback())
	end
	local style = {}
	setmetatable(style, { __index = parent })
	for k,v in pairs(value or {}) do
		if type(v) == "table" and type(parent[k]) == "table" then
			-- recursively inherrit from parent style
			style[k] = _uses(parent[k], v)
		else
			style[k] = v
		end
	end

	return style
end

--function displayName(self)
--	return "Clock (NEW)"
--end

Clock  = oo.class()

function Clock:notify_playerAlarmState(player, alarmSet)
	if not player:isLocal() then
		return
	end
	self.alarmSet = alarmSet

	self:Draw()
end

function Clock:__init(skin, windowStyle)
	log:debug("Init Clock")

	local obj = oo.rawnew(self)
	obj.screen_width, obj.screen_height = Framework:getScreenSize()

	-- the player object needs adding here for the alarm icon support
	obj.player = Player:getLocalPlayer()
	if obj.player then
		jnt:subscribe(obj)
		obj.alarmSet = obj.player:getAlarmState()
	else
		obj.alarmSet = nil
	end

	-- create window and icon
	if not windowStyle then
		windowStyle = 'Clock'
	end
	obj.window = Window(windowStyle)
	obj.window:setSkin(skin)
	obj.window:reSkin()
	obj.window:setShowFrameworkWidgets(false)

	obj.window:addListener(EVENT_MOTION,
		function()
			obj.window:hide(Window.transitionNone)
			return EVENT_CONSUME
		end)

	-- register window as a screensaver
	local manager = appletManager:getAppletInstance("ScreenSavers")
	manager:screensaverWindow(obj.window, _, _, _, 'Clock')

	return obj
end

--[[
function Clock:_getHour(time)
	local theHour = time.hour
	if self.clock_format_hour == "%I" then
		theHour = time.hour % 12
		if theHour == 0 then
			theHour = 12
		end
	end
	return self:_padString(theHour)

end
--]]

--[[
function Clock:_padString(number)
	if number < 10 then
		return "0" .. tostring(number)
	else
		return tostring(number)
	end
end
--]]

--[[
function Clock:_getMinute(time)
	return self:_padString(time.min)
end
--]]

--[[
function Clock:_getDate(time)
	local theDate
	if self.clock_format_date == "%d%m%Y" then
		theDate = self:_padString(time.day) .. self:_padString(time.month) .. tostring(time.year)
		
	else
		theDate = self:_padString(time.month) .. self:_padString(time.day) .. tostring(time.year)
	end
	return theDate
end
--]]

Word = oo.class({}, Clock)

function Word:__init(applet)
	log:info("Init Word Clock")

	local skinName   = jiveMain:getSelectedSkin()
	log:info("skinName = " .. skinName)
  
  Word:overrideSkinName(skinName)

	if not self.skin and self.skinName ~= self.oldSkinName then
  	log:info("Fetching ClockSkin")
		self.oldSkinName = self.skinName
		self.skin = Word:getWordClockSkin(self.skinName)
	end
  
	obj = oo.rawnew(self, Clock(self.skin))
	obj.textdate = Label('textdate')

	obj.skinParams = Word:getSkinParams(self.skinName)

  obj.pointer_textIt         = Surface:loadImage(obj.skinParams.textIt)  
  obj.pointer_textIs         = Surface:loadImage(obj.skinParams.textIs)  
  obj.pointer_textHas        = Surface:loadImage(obj.skinParams.textHas)  
  obj.pointer_textNearly     = Surface:loadImage(obj.skinParams.textNearly)  
  obj.pointer_textJustgone   = Surface:loadImage(obj.skinParams.textJustgone)  

  obj.pointer_textHalf       = Surface:loadImage(obj.skinParams.textHalf)  
  obj.pointer_textTen        = Surface:loadImage(obj.skinParams.textTen)  
  obj.pointer_textAquarter   = Surface:loadImage(obj.skinParams.textAQuarter)  
  obj.pointer_textTwenty     = Surface:loadImage(obj.skinParams.textTwenty)  

  obj.pointer_textFive       = Surface:loadImage(obj.skinParams.textFive)  
  obj.pointer_textMinutes    = Surface:loadImage(obj.skinParams.textMinutes)  
  obj.pointer_textTo         = Surface:loadImage(obj.skinParams.textTo)  
  obj.pointer_textPast       = Surface:loadImage(obj.skinParams.textPast)  

  obj.pointer_textHourOne    = Surface:loadImage(obj.skinParams.textHourOne)  
  obj.pointer_textHourTwo    = Surface:loadImage(obj.skinParams.textHourTwo)  
  obj.pointer_textHourThree  = Surface:loadImage(obj.skinParams.textHourThree)  
  obj.pointer_textHourFour   = Surface:loadImage(obj.skinParams.textHourFour)  
  obj.pointer_textHourFive   = Surface:loadImage(obj.skinParams.textHourFive)  
  obj.pointer_textHourSix    = Surface:loadImage(obj.skinParams.textHourSix)  
  obj.pointer_textHourSeven  = Surface:loadImage(obj.skinParams.textHourSeven)  
  obj.pointer_textHourEight  = Surface:loadImage(obj.skinParams.textHourEight)  
  obj.pointer_textHourNine   = Surface:loadImage(obj.skinParams.textHourNine)  
  obj.pointer_textHourTen    = Surface:loadImage(obj.skinParams.textHourTen)  
  obj.pointer_textHourEleven = Surface:loadImage(obj.skinParams.textHourEleven)  
  obj.pointer_textHourTwelve = Surface:loadImage(obj.skinParams.textHourTwelve)  

  obj.pointer_textOClock     = Surface:loadImage(obj.skinParams.textOClock)  
  obj.pointer_textAM         = Surface:loadImage(obj.skinParams.textAM)  
  obj.pointer_textPM         = Surface:loadImage(obj.skinParams.textPM)  

	obj.alarmIcon = Surface:loadImage(obj.skinParams.alarmIcon)

	-- bring in applet's self so strings are available
	obj.applet    = applet

	obj.canvas   = Canvas('debug_canvas', function(screen)
		obj:_reDraw(screen)
	end)
	obj.window:addWidget(obj.canvas)
	obj.window:addWidget(obj.textdate)

	obj.clock_format = "%H:%M"
	return obj
end


function Word:Draw()
	log:info("Word:Draw")
	self.canvas:reDraw()
end


function Word:_reDraw(screen)

	log:info("Word:_reDraw")

  timenow = os.date("*t",os.time())

  flags = Word:getwordflags(timenow)

  all = false  -- Just for debugging screen position
  
-- Row 1
  
 	self.pointer_textIt:blit(screen, 20, 50)
 	if all or flags.is         then self.pointer_textIs:blit(screen, 86, 50) end
 	if all or flags.has        then self.pointer_textHas:blit(screen, 156, 50) end
 	if all or flags.nearly     then self.pointer_textNearly:blit(screen, 280, 50) end
 	if all or flags.justgone   then self.pointer_textJustgone:blit(screen, 496, 50) end

-- Row 2

 	if all or flags.half       then self.pointer_textHalf:blit(screen, 20, 108) end
 	if all or flags.ten        then self.pointer_textTen:blit(screen, 163, 108) end
 	if all or flags.aquarter   then self.pointer_textAquarter:blit(screen, 274, 108) end
 	if all or flags.twenty     then self.pointer_textTwenty:blit(screen, 579, 108) end

-- Row 3

 	if all or flags.five       then self.pointer_textFive:blit(screen, 20, 165) end
 	if all or flags.minutes    then self.pointer_textMinutes:blit(screen, 169, 165) end
 	if all or flags.to         then self.pointer_textTo:blit(screen, 425, 165) end
 	if all or flags.past       then self.pointer_textPast:blit(screen, 537, 165) end
 	if all or flags.hsix       then self.pointer_textHourSix:blit(screen, 707, 165) end

-- Row 4

 	if all or flags.hseven     then self.pointer_textHourSeven:blit(screen, 20, 222) end
 	if all or flags.hone       then self.pointer_textHourOne:blit(screen, 222, 222) end
 	if all or flags.htwo       then self.pointer_textHourTwo:blit(screen, 363, 222) end
 	if all or flags.hten       then self.pointer_textHourTen:blit(screen, 513, 222) end
 	if all or flags.hfour      then self.pointer_textHourFour:blit(screen, 650, 222) end

-- Row 5

 	if all or flags.hfive      then self.pointer_textHourFive:blit(screen, 20, 280) end
 	if all or flags.hnine      then self.pointer_textHourNine:blit(screen, 193, 280) end
 	if all or flags.htwelve    then self.pointer_textHourTwelve:blit(screen, 371, 280) end
 	if all or flags.height     then self.pointer_textHourEight:blit(screen, 639, 280) end

-- Row 6

 	if all or flags.heleven    then self.pointer_textHourEleven:blit(screen, 20, 338) end
 	if all or flags.hthree     then self.pointer_textHourThree:blit(screen, 222, 338) end
 	if all or flags.oclock     then self.pointer_textOClock:blit(screen, 398, 338) end
 	if all or flags.am         then self.pointer_textAM:blit(screen, 627, 338) end
 	if all or flags.pm         then self.pointer_textPM:blit(screen, 716, 338) end

  self.textdate:setValue("ON " .. string.upper(Word:getDateAsWords(tonumber(os.date("%d")))))

	if self.alarmSet then
		local tmp = self.alarmIcon
		tmp:blit(screen, self.skinParams.alarmX, self.skinParams.alarmY)
	end
  
end


-- keep these methods with their legacy names
-- to ensure backwards compatibility with old settings.lua files
function openWordClock(self, force)
	return self:_openScreensaver("Word", _, force)
end

function _tick(self)
	local theTime = os.date(self.clock.clock_format)
	if theTime == self.oldTime then
		-- nothing to do yet
		return
	end

	self.oldTime = theTime

	if not self.snapshot then
		self.snapshot = SnapshotWindow()
		local manager = appletManager:getAppletInstance("ScreenSavers")
		manager:screensaverWindow(self.snapshot, _, _, _, 'Clock')
	else
		self.snapshot:refresh()
	end
	self.snapshot:replace(self.clock.window)
	self.clock:Draw()
	self.clock.window:replace(self.snapshot, Window.transitionFadeIn)

end


function _openScreensaver(self, type, windowStyle, force)
	log:info("Type: " .. type)

	local year = os.date("%Y")
	if tonumber(year) < 2009 and not force then
		local time = os.date()
		log:warn('This device does not seem to have the right time: ', time)
		return
	end
	-- Global Date/Time Settings
	local weekstart       = datetime:getWeekstart() 
	local hours           = datetime:getHours() 
	local shortDateFormat = datetime:getShortDateFormat() 

	hours = (hours == "12")

	-- Create two clock instances, so that we can do use a fade in transition

	if type == "Word" then
		self.clock = Word(self)
	else
		log:error("Unknown clock type")
		return
	end

	self.clock.window:addTimer(1000, function() self:_tick() end)

	self.clock:Draw()
	self.clock.window:show(Window.transitionFadeIn)

	return true
end

-- WORD CLOCK
function Word:overrideSkinName(skinName)

	if skinName == 'WVGAsmallSkin' or skinName == 'JogglerSkin' then
    skinName = 'WVGASkin'
  elseif skinName == 'WQVGAsmallSkin' or skinName == 'WQVGAlargeSkin' then
    skinName = 'WQVGASkin'
  end
   
	self.skinName = skinName

end

function Word:getWordClockSkin(skinName)

	log:info("Word:getWordClockSkin - " .. skinName)

	self.imgpath = _imgpath(self)

	log:info("Image path - " .. self.imgpath)
	local s = {}

	local wordClockBackground
	
	if skinName == 'WVGASkin' then
  
		wordClockBackground = Tile:loadImage(self.imgpath .. "wallpaper_clock_word.png")
    
    s.Clock = {
      bgImg = wordClockBackground,
      textdate = {
        position = LAYOUT_NONE,
        x = 0,
        y = 420,
        w = 800,
        font = _font(26),
        align = 'bottom',
        fg = { 0xff, 0xff, 0xff },
      },
    }

	elseif skinName == 'WQVGASkin' then
  
		wordClockBackground = Tile:loadImage(self.imgpath .. "wallpaper_clock_word.png")
    
    s.Clock = {
      bgImg = wordClockBackground,
      textdate = {
        position = LAYOUT_NONE,
        x = 10,
        y = 228,
        font = _font(22),
        align = 'bottom',
        fg = { 0xff, 0xff, 0xff },
      },
    }

	end

	return s

end


function Word:getSkinParams(skinName)

	log:info("Word:getSkinParams - " .. skinName)

  -- WVGA = 800x480 (Joggler res)
  -- WQVGA = 400x240
  
	if skinName == 'WVGASkin' then
    return {
      textIt        = self.imgpath .. 'text-it.png',
      textIs        = self.imgpath .. 'text-is.png',
      textHas       = self.imgpath .. 'text-has.png',
      textNearly    = self.imgpath .. 'text-nearly.png',
      textJustgone  = self.imgpath .. 'text-justgone.png',

      textHalf      = self.imgpath .. 'text-half.png',
      textTen       = self.imgpath .. 'text-ten.png',
      textAQuarter  = self.imgpath .. 'text-aquarter.png',
      textTwenty    = self.imgpath .. 'text-twenty.png',

      textFive       = self.imgpath .. 'text-five.png',
      textMinutes    = self.imgpath .. 'text-minutes.png',
      textTo         = self.imgpath .. 'text-to.png',
      textPast       = self.imgpath .. 'text-past.png',
      
      textHourOne    = self.imgpath .. 'text-hour-one.png',
      textHourTwo    = self.imgpath .. 'text-hour-two.png',
      textHourThree  = self.imgpath .. 'text-hour-three.png',
      textHourFour   = self.imgpath .. 'text-hour-four.png',
      textHourFive   = self.imgpath .. 'text-hour-five.png',
      textHourSix    = self.imgpath .. 'text-hour-six.png',
      textHourSeven  = self.imgpath .. 'text-hour-seven.png',
      textHourEight  = self.imgpath .. 'text-hour-eight.png',
      textHourNine   = self.imgpath .. 'text-hour-nine.png',
      textHourTen    = self.imgpath .. 'text-hour-ten.png',
      textHourEleven = self.imgpath .. 'text-hour-eleven.png',
      textHourTwelve = self.imgpath .. 'text-hour-twelve.png',
      
      textOClock     = self.imgpath .. 'text-oclock.png',
      textAM         = self.imgpath .. 'text-am.png',
      textPM         = self.imgpath .. 'text-pm.png',

			alarmIcon  = self.imgpath .. 'icon_alarm_word.png',
			alarmX     = 527,
			alarmY     = 72,
		}
  elseif skinName == 'WQVGASkin' then
    return {
			alarmIcon  = self.imgpath .. 'icon_alarm_word.png',
			alarmX     = 527,
			alarmY     = 72,
		}
	end
  
end


function Word:getwordflags(timenow)

	flags = {}

	fifths = {
		function(x) flags.zero = true end,
		function(x) flags.five = true flags.minutes = true end,
		function(x) flags.ten = true flags.minutes = true end,
		function(x) flags.aquarter = true end,
		function(x) flags.twenty = true flags.minutes = true end,
		function(x) flags.twenty = true flags.five = true flags.minutes = true end,
		function(x) flags.half = true end,
		function(x) flags.twenty = true flags.five = true flags.minutes = true end,
		function(x) flags.twenty = true flags.minutes = true end,
		function(x) flags.aquarter = true end,
		function(x) flags.ten = true flags.minutes = true end,
		function(x) flags.five = true flags.minutes = true end,
		function(x) flags.zero = true end
	}

	-- Work out IS, HAS, NEARLY and JUST GONE

	tmp = timenow.min % 5

	if tmp == 0 then
	  flags.is = true flags.exactly = true
	elseif tmp == 1 or tmp == 2 then
	  flags.has = true flags.justgone = true
	elseif tmp == 3 or tmp == 4 then
	  flags.is = true flags.nearly = true
	end

	-- Work out five minute divisions

	tmp = math.floor(timenow.min / 5) + 2

	if flags.exactly or flags.justgone then
	  tmp = tmp - 1
	end

	fifths[tmp]()

	-- Work out TO, PAST and OCLOCK

	if (timenow.min >= 0 and timenow.min <= 2) or timenow.min == 58 or timenow.min == 59 then
	  flags.oclock = true
	elseif (timenow.min >= 3 and timenow.min <= 32) then
	  flags.past = true
	elseif (timenow.min >= 33 and timenow.min <= 57) then
	  flags.to = true
	end

	-- Work out whether AM or PM

	  if timenow.hour <= 11 then
		flags.am = true
	  else
		flags.pm = true
	  end

	-- Work out hour

	hours = {
		function(x) flags.htwelve = true end,
		function(x) flags.hone = true end,
		function(x) flags.htwo = true end,
		function(x) flags.hthreee = true end,
		function(x) flags.hfour = true end,
		function(x) flags.hfive = true end,
		function(x) flags.hsix = true end,
		function(x) flags.hseven = true end,
		function(x) flags.height = true end,
		function(x) flags.hnine = true end,
		function(x) flags.hten = true end,
		function(x) flags.heleven = true end,
		function(x) flags.htwelve = true end
	}

	hour12 = (timenow.hour % 12) + 1

	if timenow.min >= 0 and timenow.min <=32 then
	  hours[hour12]()
	elseif timenow.min >= 33 and timenow.min <=59 then
	  hours[hour12 + 1]()
	end

	return flags

end

-- Generate output

function Word:timeastext(flags)

	timestr = "IT"

	if flags.is == true then
	  timestr = timestr .. " IS"
	end

	if flags.has == true then
	  timestr = timestr .. " HAS"
	end

	if flags.justgone == true then
	  timestr = timestr .. " JUST GONE"
	end

	if flags.nearly == true then
	  timestr = timestr .. " NEARLY"
	end

	if flags.ten == true then
	  timestr = timestr .. " TEN"
	elseif flags.aquarter == true then
	  timestr = timestr .. " A QUARTER"
	elseif flags.twenty == true then
	  timestr = timestr .. " TWENTY"
	end

	if flags.five == true then
	  timestr = timestr .. " FIVE"
	end

	if flags.half == true then
	  timestr = timestr .. " HALF"
	end

	if flags.past == true then
	  timestr = timestr .. " PAST"
	end

	if flags.to == true then
	  timestr = timestr .. " TO"
	end

	if flags.hone == true then
	  timestr = timestr .. " ONE"
	elseif flags.htwo == true then
	  timestr = timestr .. " TWO"
	elseif flags.hthree == true then
	  timestr = timestr .. " THREE"
	elseif flags.hfour == true then
	  timestr = timestr .. " FOUR"
	elseif flags.hfive == true then
	  timestr = timestr .. " FIVE"
	elseif flags.hsix == true then
	  timestr = timestr .. " SIX"
	elseif flags.hseven == true then
	  timestr = timestr .. " SEVEN"
	elseif flags.height == true then
	  timestr = timestr .. " EIGHT"
	elseif flags.hnine == true then
	  timestr = timestr .. " NINE"
	elseif flags.hten == true then
	  timestr = timestr .. " TEN"
	elseif flags.heleven == true then
	  timestr = timestr .. " ELEVEN"
	elseif flags.htwelve == true then
	  timestr = timestr .. " TWELVE"
	end

	if flags.oclock == true then
	  timestr = timestr .. " O'CLOCK"
	end

	if flags.am == true then
	  timestr = timestr .. " AM"
	elseif flags.pm == true then
	  timestr = timestr .. " PM"
	end

	return timestr

end

function Word:getDateAsWords(day)

	w = {
		"First",
		"Second",
		"Third",
		"Fourth",
		"Fifth",
		"Sixth",
		"Seventh",
		"Eighth",
		"Ninth",
		"Tenth",
		"Eleventh",
		"Twelfth",
		"Thirteenth",
		"Fourteenth",
		"Fifteenth",
		"Sixteenth",
		"Seventeenth",
		"Eighteenth",
		"Nineteenth",
		"Twentieth",
		"Twenty First",
		"Twenty Second",
		"Twenty Third",
		"Twenty Fourth",
		"Twenty Fifth",
		"Twenty Sixth",
		"Twenty Seventh",
		"Twenty Eighth",
		"Twenty Ninth",
		"Thirtieth",
		"Thirty First"
	}

    return( os.date("%A the ") .. w[day] .. " of " .. os.date("%B"))

end


--[[

=head1 LICENSE

Copyright 2010 Logitech. All Rights Reserved.

This file is licensed under BSD. Please see the LICENSE file for details.

=cut
--]]
